function* idGenerator() {
  let i = 1;
  while (i < Number.MAX_SAFE_INTEGER) {
    yield i++;
  }
  throw "Cannot generate more ids";
}

export function makeIdGenerator() {
  const gen = idGenerator();
  return () => gen.next().value;
}

export function getQuadFromBBox(bbox = []) {
  if (!bbox || bbox.length !== 4) {
    throw "bbox have to be an array with x1, y1, x2, y2 values";
  }
  const [x1, y1, x2, y2] = bbox;
  return {
    p1: { x: x1, y: y1 },
    p2: { x: x2, y: y1 },
    p3: { x: x2, y: y2 },
    p4: { x: x1, y: y2 },
  };
}

export function capitalCaseFirstLetter(str = "") {
  return str[0].toUpperCase() + str.slice(1);
}

/******* COLORS *******/

// after this value, the color restart to rgb(0,0,0)
const maxColorId = 838860799;
// a step should be added to prevent antialiasing to leads to wrong id detection
const colorStep = 50;

export function getRGBStringColorFromId(id = 0) {
  if (id < 0 || id * colorStep > maxColorId) {
    throw `Impossible to generate unique color from id ${id}.`;
  }
  const rgb = [];
  rgb.push((id * colorStep) & 0xff); // R.
  rgb.push(((id * colorStep) & 0xff00) >> 8); // G.
  rgb.push(((id * colorStep) & 0xff0000) >> 16); // B.
  return `rgb(${rgb.join(",")})`;
}

export const getIdFromRGBColor = function (color) {
  if (!color || color.length < 3) {
    throw "color must be an array with at least 3 values.";
  }
  const R = color[0];
  const G = color[1];
  const B = color[2];

  return R / colorStep + (G * 256) / colorStep + (B * 256 * 256) / colorStep;
};

export function fromUint8ArrayColorToStringColor(color, opacity = 1) {
  return `rgba(${color[0]},${color[1]},${color[2]},${opacity})`;
}
